#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "kernelsolver.h"
#include <QMessageBox>

const int max_n = 4096;

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), ui(new Ui::MainWindow)
{
  ui->setupUi(this);

  ui->tableWidget->setColumnWidth(0, 160);
  ui->tableWidget->setColumnWidth(1, 250);
  ui->tableWidget->setColumnWidth(2, 240);
  ui->tableWidget->setColumnWidth(3, 230);

  int Width = 0;
  for (int i = 0; i < ui->tableWidget->columnCount(); i++) {
    Width += ui->tableWidget->columnWidth(i);
  }

  this->resize(Width + 40, this->height());

  QDesktopWidget desktop;
  QRect rect = desktop.availableGeometry(desktop.primaryScreen());
  QPoint center = rect.center();
  center.setX(center.x() - (this->width()/2));
  center.setY(center.y() - (this->height()/2));
  move(center);

  ui->label->move(width()/2 - ui->label->geometry().width()/2, 10);

  ui->tableWidget->resize(Width + 10, ui->tableWidget->height() + 10);
  ui->tableWidget->move(width()/2 - ui->tableWidget->geometry().width()/2, ui->label->geometry().y() + ui->label->geometry().height() + 10);

  ui->label_2->setVisible(false);
  ui->label_2->move(ui->tableWidget->geometry().x(), ui->tableWidget->geometry().y() + ui->tableWidget->geometry().height() + 10);

  ui->progressBar->setMaximum(max_n);
  ui->progressBar->setVisible(false);
  ui->progressBar->move(ui->label_2->geometry().x(), ui->label_2->geometry().y() + ui->label_2->geometry().height() + 20);
  ui->progressBar->resize(ui->tableWidget->width(), ui->progressBar->height());

  ui->pushButton->move(width()/2 - ui->pushButton->geometry().width()/2, ui->progressBar->geometry().y() + ui->progressBar->geometry().height() + 10);

  int deviceCount;
  cudaDeviceProp devProp;

  cudaGetDeviceCount(&deviceCount);
  int len = ui->comboBox->geometry().width();

  for (int i = 0; i < deviceCount; i++){
    cudaGetDeviceProperties(&devProp, i);
    QString name = devProp.name;

    len = max(len, name.size() + 200);
    ui->comboBox->addItem(name);
  }

  ui->comboBox->setCurrentIndex(-1);
  ui->comboBox->resize(len, ui->comboBox->height());

  ui->comboBox->move(width() - ui->comboBox->width() - 10, ui->comboBox->geometry().y());

  ui->label_3->move(ui->comboBox->geometry().x() - ui->label_3->width() - 3, ui->comboBox->geometry().y());

  if(deviceCount == 0) {
    QMessageBox::information(this, "Ошибка", "Выбирете устройство для расчета");
    ui->pushButton->setEnabled(false);
  }
}

MainWindow::~MainWindow()
{
  delete ui;
}

void MainWindow::AddOut(int Size){
  QProcess* p = new QProcess(this);
  QStringList arg;

  arg << "matrix_matrix.cpp" << "-o" << "matrix_matrix";
  p->start("mpic++", arg);
  p->waitForFinished(-1);

  QProcess* p1 = new QProcess(this);
  QStringList arg1;

  QString size;
  size.sprintf("%d", Size);
  arg1 << "matrix_matrix" << size;
  p1->start("./bat.sh", arg1);
  p1->waitForFinished(-1);
}

void MainWindow::SetTableWidget(int row, int column, double text){
  QString Text = "";

  if(text != 0){
    if(column == 0) {
      Text.sprintf("%.0f",text);
    }else {
      Text.sprintf("%.5f",text);
    }
  }

  ui->tableWidget->item(row, column)->setText(Text);
  ui->tableWidget->item(row, column)->setTextAlignment(ui->tableWidget->horizontalHeaderItem(column)->textAlignment());
}

void MainWindow::on_pushButton_clicked()
{
  if(ui->comboBox->currentIndex() == -1){
    QMessageBox::critical(this, "Ошибка", "Выбирете устройство для расчета");
    return;
  }

  cudaSetDevice(ui->comboBox->currentIndex());

  for (int i = 0; i < ui->tableWidget->rowCount(); i++){
    for (int j = 0; j < ui->tableWidget->columnCount(); j++){
      ui->tableWidget->item(i, j)->setText("");
    }
  }

  ui->pushButton->setEnabled(false);

  ui->label_2->setVisible(true);
  ui->progressBar->setVisible(true);

  ui->label_2->move(ui->tableWidget->geometry().x(), ui->tableWidget->geometry().y() + ui->tableWidget->geometry().height() + 10);
  ui->progressBar->move(ui->label_2->geometry().x(), ui->label_2->geometry().y() + ui->label_2->geometry().height() + 20);

  ui->progressBar->setValue(0);

  freopen("out.txt", "w+", stdin);

  double *matrixA, *matrixB, *matrixResult;
  for (int n = 16, k = 0; n <= max_n; n<<=1, k++){
    processInitialization(matrixA, matrixB, matrixResult, n);

    SetTableWidget(k, 0, n);
    SetTableWidget(k, 1, solve(matrixA, matrixB, matrixResult, n));
    SetTableWidget(k, 2, solve_cublas(matrixA, matrixB, matrixResult, n));

    AddOut(n);
    double MPITime;
    cin>>MPITime;
    SetTableWidget(k, 3, MPITime);
    processTermination(matrixA, matrixB, matrixResult);

    ui->progressBar->setValue(n);
  }

  ui->pushButton->setEnabled(true);
  ui->label_2->setVisible(false);
  ui->progressBar->setVisible(false);
}
